@@ -9,20 +9,36 @@
 #include <algorithm>
 
 /**
- * Calculate the one-dimensional index for a specified position in a 2D grid.
+ * Converts a 2D grid position (row, column) to a 1D index in row-major order.
+ * This function calculates the corresponding one-dimensional index based on the given
+ * row and column position within a grid of specified dimensions.
  *
- * This function converts a row and column position in a 2D grid into a corresponding
- * one-dimensional index, assuming the grid is stored in row-major order (i.e., rows are stored sequentially).
+ * Parameters:
+ *   curRow   - The current row number (1-based indexing).
+ *   curCol   - The current column number (1-based indexing).
+ *   totalRow - The total number of rows in the grid.
+ *   totalCol - The total number of columns in the grid.
  *
- * @param curRow   The current row number (1-based indexing).
- * @param curCol   The current column number (1-based indexing).
- * @param totalRow The total number of rows in the grid.
- * @param totalCol The total number of columns in the grid.
- * @return         The calculated one-dimensional index (0-based indexing).
+ * Returns:
+ *   The computed 1D index (0-based indexing) corresponding to the input row and column.
  */
 int LocateGridIndices(int curRow, int curCol,
                       int totalRow, int totalCol);
 
+/**
+ * Converts a 1D grid index (cell number) to a 2D grid position (row, column) in row-major order.
+ * This function determines the corresponding row and column indices based on a given
+ * one-dimensional index within a grid of specified dimensions.
+ *
+ * Parameters:
+ *   cellNum  - The 1D index of the cell (0-based indexing).
+ *   totalCol - The total number of columns in the grid.
+ *
+ * Returns:
+ *   A std::vector<int> containing the row index and column index (both 0-based indexing).
+ */
+std::vector<int> RowColFromGrid(int cellNum, int totalCol);
+
 /**
  * Converts a 2D grid data matrix (vector of vectors) to a 1D vector by concatenating the rows.
  * This function iterates over each row of the input matrix and appends the elements to the resulting vector.
@@ -81,16 +97,18 @@ std::vector<std::vector<double>> CppLaggedVar4Grid(
  *   tau  - The spatial lag step for constructing lagged state-space vectors.
  *
  * Returns:
- *   A 2D vector (matrix) where each row contains the original value (if includeself is true)
- *   and the averaged lagged variables for each embedding dimension (column).
+ *   A 2D vector (matrix) where each row contains the averaged lagged variables for
+ *   each embedding dimension (column). Columns where all values are NaN are removed.
  *
- * If includeself is true, the first column will contain the original values from mat,
- * and the subsequent columns will contain averaged lagged variables computed using the specified lag numbers.
- * If includeself is false, the matrix will only contain the averaged lagged variables.
+ * Note:
+ *   When tau = 0, the lagged variables are calculated for lag steps of 0, 1, ..., E-1.
+ *   When tau > 0, the lagged variables are calculated for lag steps of tau, 2*tau, ..., E*tau,
+ *   and this means the actual lag steps form an arithmetic sequence with a common difference of tau.
  */
 std::vector<std::vector<double>> GenGridEmbeddings(
     const std::vector<std::vector<double>>& mat,
     int E,
-    int tau);
+    int tau
+);
 
 #endif // CppGridUtils_H
