@@ -2,11 +2,12 @@
 #define CppStats_H
 
 #include <iostream>
+#include <stdexcept>
 #include <vector>
 #include <cmath>
-#include <stdexcept>
 #include <numeric> // for std::accumulate
 #include <limits>  // for std::numeric_limits
+#include "DeLongPlacements.h"
 // #include <Rcpp.h>
 #include <RcppArmadillo.h>
 
@@ -14,6 +15,14 @@ bool isNA(double value);
 
 bool checkIntNA(int value);
 
+bool checkOneDimVectorHasNaN(const std::vector<double>& vec);
+
+int checkOneDimVectorNotNanNum(const std::vector<double>& vec);
+
+unsigned long long CppFactorial(unsigned int n);
+
+unsigned long long CppCombine(unsigned int n, unsigned int k);
+
 double CppMean(const std::vector<double>& vec,
                bool NA_rm = false);
 
@@ -28,23 +37,22 @@ double CppRMSE(const std::vector<double>& x1,
                const std::vector<double>& x2,
                bool NA_rm = false);
 
+std::vector<double> CppCumSum(const std::vector<double>& vec);
+
 std::vector<double> CppAbsDiff(const std::vector<double>& vec1,
                                const std::vector<double>& vec2);
 
 std::vector<double> CppSumNormalize(const std::vector<double>& vec,
                                     bool NA_rm = false);
 
+std::vector<double> CppArithmeticSeq(double from, double to, int length_out);
+
 double CppVariance(const std::vector<double>& vec, bool NA_rm = false);
 
 double CppCovariance(const std::vector<double>& vec1,
                      const std::vector<double>& vec2,
                      bool NA_rm = false);
 
-double CppDistance(const std::vector<double>& vec1,
-                   const std::vector<double>& vec2,
-                   bool L1norm = false,
-                   bool NA_rm = false);
-
 double PearsonCor(const std::vector<double>& y,
                   const std::vector<double>& y_hat,
                   bool NA_rm = false);
@@ -66,11 +74,41 @@ double CppCorSignificance(double r, int n, int k = 0);
 std::vector<double> CppCorConfidence(double r, int n, int k = 0,
                                      double level = 0.05);
 
+std::vector<double> CppDeLongAUCConfidence(const std::vector<double>& cases,
+                                           const std::vector<double>& controls,
+                                           const std::string& direction,
+                                           double level = 0.05);
+
+std::vector<double> CppCMCTest(const std::vector<double>& cases,
+                               const std::string& direction,
+                               double level = 0.05,
+                               int num_samples = 0);
+
+std::vector<double> CppDeLongTest(const std::vector<double>& cases,
+                                  const std::vector<double>& controls,
+                                  const std::string& direction,
+                                  double level = 0.05);
+
+double CppDistance(const std::vector<double>& vec1,
+                   const std::vector<double>& vec2,
+                   bool L1norm = false,
+                   bool NA_rm = false);
+
+std::vector<std::vector<double>> CppMatDistance(
+    const std::vector<std::vector<double>>& mat,
+    bool L1norm = false,
+    bool NA_rm = false);
+
 std::vector<std::size_t> CppKNNIndice(
     const std::vector<std::vector<double>>& embedding_space,
     std::size_t target_idx,
     std::size_t k);
 
+std::vector<std::size_t> CppDistKNNIndice(
+    const std::vector<std::vector<double>>& dist_mat,
+    std::size_t target_idx,
+    std::size_t k);
+
 std::vector<std::vector<std::vector<double>>> CppSVD(const std::vector<std::vector<double>>& X);
 
 std::vector<double> LinearTrendRM(const std::vector<double>& vec,
