@@ -20,7 +20,6 @@
  * Parameters:
  *   - x_vectors: Reconstructed state-space (each row represents a separate vector/state).
  *   - y: Spatial cross-section series used as the target (should align with x_vectors).
- *   - lib_indices: A boolean vector indicating which states to include when searching for neighbors.
  *   - lib_size: Size of the library used for cross mapping.
  *   - max_lib_size: Maximum size of the library.
  *   - possible_lib_indices: Indices of possible library states.
@@ -28,6 +27,8 @@
  *   - b: Number of neighbors to use for simplex projection.
  *   - simplex: If true, uses simplex projection for prediction; otherwise, uses s-mapping.
  *   - theta: Distance weighting parameter for local neighbors in the manifold (used in s-mapping).
+ *   - threads: The number of threads to use for parallel processing.
+ *   - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  *
  * Returns:
  *   A vector of pairs, where each pair consists of:
@@ -37,14 +38,15 @@
 std::vector<std::pair<int, double>> GCCMSingle4Lattice(
     const std::vector<std::vector<double>>& x_vectors,
     const std::vector<double>& y,
-    const std::vector<bool>& lib_indices,
     int lib_size,
     int max_lib_size,
     const std::vector<int>& possible_lib_indices,
     const std::vector<bool>& pred_indices,
     int b,
     bool simplex,
-    double theta
+    double theta,
+    size_t threads,
+    int parallel_level
 );
 
 /**
@@ -63,6 +65,7 @@ std::vector<std::pair<int, double>> GCCMSingle4Lattice(
  * - simplex: Boolean flag indicating whether to use simplex projection (true) or S-mapping (false) for prediction.
  * - theta: Distance weighting parameter used for weighting neighbors in the S-mapping prediction.
  * - threads: Number of threads to use for parallel computation.
+ * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - progressbar: Boolean flag to indicate whether to display a progress bar during computation.
  *
  * Returns:
@@ -70,8 +73,8 @@ std::vector<std::pair<int, double>> GCCMSingle4Lattice(
  *      - The library size.
  *      - The mean cross-mapping correlation.
  *      - The statistical significance of the correlation.
- *      - The lower bound of the confidence interval.
  *      - The upper bound of the confidence interval.
+ *      - The lower bound of the confidence interval.
  */
 std::vector<std::vector<double>> GCCM4Lattice(
     const std::vector<double>& x,
@@ -86,6 +89,7 @@ std::vector<std::vector<double>> GCCM4Lattice(
     bool simplex,
     double theta,
     int threads,
+    int parallel_level,
     bool progressbar
 );
 
