@@ -27,7 +27,7 @@
  * @param pred_indices: Boolean vector indicating which states to use for predictions.
  * @param conEs: Vector specifying the number of dimensions for attractor reconstruction with control variables.
  * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
- * @param num_neighbors: Number of neighbors to use for simplex projection.
+ * @param num_neighbors: Vector specifying the numbers of neighbors to use for simplex projection.
  * @param cumulate: Flag indicating whether to cumulatively incorporate control variables.
  *
  * @return A std::vector<double> containing:
@@ -43,11 +43,11 @@ std::vector<double> PartialSimplex4Lattice(
     const std::vector<bool>& pred_indices,
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
-    int num_neighbors,
+    const std::vector<int>& num_neighbors,
     bool cumulate
 ){
   int n_controls = controls.size();
-  std::vector<double> rho(2);
+  std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
 
   if (cumulate) {
     std::vector<double> temp_pred;
@@ -55,33 +55,39 @@ std::vector<double> PartialSimplex4Lattice(
 
     for (int i = 0; i < n_controls; ++i) {
       if (i == 0){
-        temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors);
+        temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0]);
       } else {
-        temp_pred = SimplexProjectionPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors);
+        temp_pred = SimplexProjectionPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors[i]);
       }
       temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
     }
 
-    std::vector<double> con_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors);
-    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors);
+    std::vector<double> con_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[n_controls]);
+    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0]);
+
+    if (checkOneDimVectorNotNanNum(target_pred) >= 3){
+      rho[0] = PearsonCor(target,target_pred,true);
+      rho[1] = PartialCorTrivar(target,target_pred,con_pred,true,false);
+    }
 
-    rho[0] = PearsonCor(target,target_pred,true);
-    rho[1] = PartialCorTrivar(target,target_pred,con_pred,true,false);
   } else {
     std::vector<std::vector<double>> con_pred(n_controls);
     std::vector<double> temp_pred;
     std::vector<std::vector<double>> temp_embedding;
 
     for (int i = 0; i < n_controls; ++i) {
-      temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors);
+      temp_pred = SimplexProjectionPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0]);
       temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
-      temp_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors);
+      temp_pred = SimplexProjectionPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[i+1]);
       con_pred[i] = temp_pred;
     }
-    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors);
+    std::vector<double> target_pred = SimplexProjectionPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0]);
+
+    if (checkOneDimVectorNotNanNum(target_pred) >= 3){
+      rho[0] = PearsonCor(target,target_pred,true);
+      rho[1] = PartialCor(target,target_pred,con_pred,true,false);
+    }
 
-    rho[0] = PearsonCor(target,target_pred,true);
-    rho[1] = PartialCor(target,target_pred,con_pred,true,false);
   }
   return rho;
 }
@@ -102,7 +108,7 @@ std::vector<double> PartialSimplex4Lattice(
  * @param pred_indices: Boolean vector indicating which states to predict from.
  * @param conEs: Vector specifying the number of dimensions for attractor reconstruction with control variables.
  * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
- * @param num_neighbors: Number of neighbors to use for S-Map prediction.
+ * @param num_neighbors: Vector specifying the numbers of neighbors to use for S-Map prediction.
  * @param theta: Weighting parameter for distances in S-Map.
  * @param cumulate: Boolean flag to determine whether to cumulate the partial correlations.
  * @return A vector of size 2 containing:
@@ -118,12 +124,12 @@ std::vector<double> PartialSMap4Lattice(
     const std::vector<bool>& pred_indices,
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
-    int num_neighbors,
+    const std::vector<int>& num_neighbors,
     double theta,
     bool cumulate
 ){
   int n_controls = controls.size();
-  std::vector<double> rho(2);
+  std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
 
   if (cumulate){
     std::vector<double> temp_pred;
@@ -131,33 +137,39 @@ std::vector<double> PartialSMap4Lattice(
 
     for (int i = 0; i < n_controls; ++i) {
       if (i == 0){
-        temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors, theta);
+        temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], theta);
       } else {
-        temp_pred = SMapPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors, theta);
+        temp_pred = SMapPrediction(temp_embedding, controls[i], lib_indices, pred_indices, num_neighbors[i], theta);
       }
       temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
     }
 
-    std::vector<double> con_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors, theta);
-    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors, theta);
+    std::vector<double> con_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[n_controls], theta);
+    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], theta);
+
+    if (checkOneDimVectorNotNanNum(target_pred) >= 3){
+      rho[0] = PearsonCor(target,target_pred,true);
+      rho[1] = PartialCorTrivar(target,target_pred,con_pred,true,false);
+    }
 
-    rho[0] = PearsonCor(target,target_pred,true);
-    rho[1] = PartialCorTrivar(target,target_pred,con_pred,true,false);
   } else {
     std::vector<std::vector<double>> con_pred(n_controls);
     std::vector<double> temp_pred;
     std::vector<std::vector<double>> temp_embedding;
 
     for (int i = 0; i < n_controls; ++i) {
-      temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors, theta);
+      temp_pred = SMapPrediction(vectors, controls[i], lib_indices, pred_indices, num_neighbors[0], theta);
       temp_embedding = GenLatticeEmbeddings(temp_pred,nb_vec,conEs[i],taus[i]);
-      temp_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors, theta);
+      temp_pred = SMapPrediction(temp_embedding, target, lib_indices, pred_indices, num_neighbors[i+1], theta);
       con_pred[i] = temp_pred;
     }
-    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors, theta);
+    std::vector<double> target_pred = SMapPrediction(vectors, target, lib_indices, pred_indices, num_neighbors[0], theta);
+
+    if (checkOneDimVectorNotNanNum(target_pred) >= 3){
+      rho[0] = PearsonCor(target,target_pred,true);
+      rho[1] = PartialCor(target,target_pred,con_pred,true,false);
+    }
 
-    rho[0] = PearsonCor(target,target_pred,true);
-    rho[1] = PartialCor(target,target_pred,con_pred,true,false);
   }
 
   return rho;
@@ -171,16 +183,17 @@ std::vector<double> PartialSMap4Lattice(
  *   - y: Spatial cross-section series used as the target (should align with x_vectors).
  *   - controls: Cross-sectional data of control variables (stored by row).
  *   - nb_vec: Neighbor indices vector of the spatial units.
- *   - lib_indices: A boolean vector indicating which states to include when searching for neighbors.
  *   - lib_size: Size of the library used for cross mapping.
  *   - max_lib_size: Maximum size of the library.
  *   - possible_lib_indices: Indices of possible library states.
  *   - pred_indices: A boolean vector indicating which states to use for prediction.
  *   - conEs: Number of dimensions for attractor reconstruction with control variables.
  *   - taus: Spatial lag step for constructing lagged state-space vectors with control variables.
- *   - b: Number of neighbors to use for simplex projection.
+ *   - b: A vector specifying the numbers of neighbors to use for simplex projection.
  *   - simplex: If true, uses simplex projection for prediction; otherwise, uses s-mapping.
  *   - theta: Distance weighting parameter for local neighbors in the manifold (used in s-mapping).
+ *   - threads: The number of threads to use for parallel processing.
+ *   - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  *   - cumulate: Whether to accumulate partial correlations.
  *
  * Returns:
@@ -194,36 +207,89 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
     const std::vector<double>& y,                       // Spatial cross-section series to be used as the target (should line up with vectors)
     const std::vector<std::vector<double>>& controls,   // Cross-sectional data of control variables (**stored by row**)
     const std::vector<std::vector<int>>& nb_vec,        // Neighbor indices vector of the spatial units
-    const std::vector<bool>& lib_indices,               // Vector of T/F values (which states to include when searching for neighbors)
     int lib_size,                                       // Size of the library
     int max_lib_size,                                   // Maximum size of the library
     const std::vector<int>& possible_lib_indices,       // Indices of possible library states
     const std::vector<bool>& pred_indices,              // Vector of T/F values (which states to predict from)
     const std::vector<int>& conEs,                      // Number of dimensions for the attractor reconstruction with control variables
     const std::vector<int>& taus,                       // Spatial lag step for constructing lagged state-space vectors with control variables
-    int b,                                              // Number of neighbors to use for simplex projection
+    const std::vector<int>& b,                          // Numbers of neighbors to use for simplex projection
     bool simplex,                                       // Algorithm used for prediction; Use simplex projection if true, and s-mapping if false
     double theta,                                       // Distance weighting parameter for the local neighbours in the manifold
+    size_t threads,                                     // Number of threads to use for parallel processing
+    int parallel_level,                                 // Level of parallel computing: 0 for `lower`, 1 for `higher`
     bool cumulate                                       // Whether to cumulate the partial correlations
 ) {
   int n = x_vectors.size();
-  std::vector<PartialCorRes> x_xmap_y;
-  std::vector<double> rho;
 
-  if (lib_size == max_lib_size) { // No possible library variation if using all vectors
+  // No possible library variation if using all vectors
+  if (lib_size == max_lib_size) {
     std::vector<bool> lib_indices(n, false);
     for (int idx : possible_lib_indices) {
       lib_indices[idx] = true;
     }
 
+    std::vector<PartialCorRes> x_xmap_y;
     // Run partial cross map and store results
+    std::vector<double> rho;
     if (simplex) {
       rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate);
     } else {
       rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, theta, cumulate);
     }
     x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
+    return x_xmap_y;
+  } else if (parallel_level == 0){
+
+    // Precompute valid indices for the library
+    std::vector<std::vector<int>> valid_lib_indices;
+    for (int start_lib = 0; start_lib < max_lib_size; ++start_lib) {
+      std::vector<int> local_lib_indices;
+      // Loop around to beginning of lib indices
+      if (start_lib + lib_size > max_lib_size) {
+        for (int i = start_lib; i < max_lib_size; ++i) {
+          local_lib_indices.emplace_back(i);
+        }
+        int num_vectors_remaining = lib_size - (max_lib_size - start_lib);
+        for (int i = 0; i < num_vectors_remaining; ++i) {
+          local_lib_indices.emplace_back(i);
+        }
+      } else {
+        for (int i = start_lib; i < start_lib + lib_size; ++i) {
+          local_lib_indices.emplace_back(i);
+        }
+      }
+      valid_lib_indices.emplace_back(local_lib_indices);
+    }
+
+    // Preallocate the result vector to avoid out-of-bounds access
+    std::vector<PartialCorRes> x_xmap_y(valid_lib_indices.size());
+
+    // Perform the operations using RcppThread
+    RcppThread::parallelFor(0, valid_lib_indices.size(), [&](size_t i) {
+      std::vector<bool> lib_indices(n, false);
+      std::vector<int> local_lib_indices = valid_lib_indices[i];
+      for(int& li : local_lib_indices){
+        lib_indices[possible_lib_indices[li]] = true;
+      }
+
+      // Run partial cross map and store results
+      std::vector<double> rho;
+      if (simplex) {
+        rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate);
+      } else {
+        rho = PartialSMap4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, theta, cumulate);
+      }
+      // Directly initialize a PartialCorRes struct with the three values
+      PartialCorRes result(lib_size, rho[0], rho[1]);
+      x_xmap_y[i] = result;
+    }, threads);
+
+    return x_xmap_y;
   } else {
+
+    std::vector<PartialCorRes> x_xmap_y;
+
     for (int start_lib = 0; start_lib < max_lib_size; ++start_lib) {
       std::vector<bool> lib_indices(n, false);
       // Setup changing library
@@ -242,6 +308,7 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
       }
 
       // Run partial cross map and store results
+      std::vector<double> rho;
       if (simplex) {
         rho = PartialSimplex4Lattice(x_vectors, y, controls, nb_vec, lib_indices, pred_indices, conEs, taus, b, cumulate);
       } else {
@@ -249,9 +316,9 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
       }
       x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
     }
-  }
 
-  return x_xmap_y;
+    return x_xmap_y;
+  }
 }
 
 /**
@@ -267,11 +334,12 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
  * - pred: A vector specifying the prediction indices (1-based in R, converted to 0-based in C++).
  * - Es: A vector specifying the embedding dimensions for attractor reconstruction using x and control variables.
  * - taus: A vector specifying the spatial lag steps for constructing lagged state-space vectors using x and control variables.
- * - b: Number of nearest neighbors used for prediction.
+ * - b: A vector specifying the numbers of nearest neighbors used for prediction.
  * - simplex: Boolean flag indicating whether to use simplex projection (true) or S-mapping (false) for prediction.
  * - theta: Distance weighting parameter used for weighting neighbors in the S-mapping prediction.
  * - threads: Number of threads to use for parallel computation.
  * - cumulate: Boolean flag indicating whether to cumulate partial correlations.
+ * - parallel_level: Level of parallel computing: 0 for `lower`, 1 for `higher`.
  * - progressbar: Boolean flag indicating whether to display a progress bar during computation.
  *
  * Returns:
@@ -279,12 +347,12 @@ std::vector<PartialCorRes> SCPCMSingle4Lattice(
  *      - The library size.
  *      - The mean pearson cross-mapping correlation.
  *      - The statistical significance of the pearson correlation.
- *      - The lower bound of the pearson correlation confidence interval.
  *      - The upper bound of the pearson correlation confidence interval.
+ *      - The lower bound of the pearson correlation confidence interval.
  *      - The mean partial cross-mapping partial correlation.
  *      - The statistical significance of the partial correlation.
- *      - The lower bound of the partial correlation confidence interval.
  *      - The upper bound of the partial correlation confidence interval.
+ *      - The lower bound of the partial correlation confidence interval.
  */
 std::vector<std::vector<double>> SCPCM4Lattice(
     const std::vector<double>& x,                       // Spatial cross-section series to cross map from
@@ -296,13 +364,22 @@ std::vector<std::vector<double>> SCPCM4Lattice(
     const std::vector<int>& pred,                       // Vector specifying the prediction indices
     const std::vector<int>& Es,                         // Number of dimensions for the attractor reconstruction with the x and control variables
     const std::vector<int>& taus,                       // Spatial lag step for constructing lagged state-space vectors with the x and control variables
-    int b,                                              // Number of nearest neighbors to use for prediction
+    const std::vector<int>& b,                          // Numbers of nearest neighbors to use for prediction
     bool simplex,                                       // Algorithm used for prediction; Use simplex projection if true, and s-mapping if false
     double theta,                                       // Distance weighting parameter for the local neighbours in the manifold
     int threads,                                        // Number of threads used from the global pool
+    int parallel_level,                                 // Level of parallel computing: 0 for `lower`, 1 for `higher`
     bool cumulate,                                      // Whether to cumulate the partial correlations
     bool progressbar                                    // Whether to print the progress bar
 ) {
+  // If b is not provided correctly, default it to E + 2
+  std::vector<int> bs = b;
+  for (size_t i = 0; i < bs.size(); ++i){
+    if (bs[i] <= 0) {
+      bs[i] = Es[i] + 2;
+    }
+  }
+
   int Ex = Es[0];
   std::vector<int> conEs = Es;
   conEs.erase(conEs.begin());
@@ -311,14 +388,9 @@ std::vector<std::vector<double>> SCPCM4Lattice(
   std::vector<int> contaus = taus;
   contaus.erase(contaus.begin());
 
-  // If b is not provided correctly, default it to Ex + 2
-  if (b <= 0) {
-    b = Ex + 2;
-  }
-
-  size_t threads_sizet = static_cast<size_t>(threads);
-  unsigned int max_threads = std::thread::hardware_concurrency();
-  threads_sizet = std::min(static_cast<size_t>(max_threads), threads_sizet);
+  // Configure threads
+  size_t threads_sizet = static_cast<size_t>(std::abs(threads));
+  threads_sizet = std::min(static_cast<size_t>(std::thread::hardware_concurrency()), threads_sizet);
 
   std::vector<std::vector<double>> x_vectors = GenLatticeEmbeddings(x,nb_vec,Ex,taux);
   int n = x_vectors.size();
@@ -330,59 +402,19 @@ std::vector<std::vector<double>> SCPCM4Lattice(
     n_confounds = controls.size();
   }
 
-  // Initialize lib_indices and pred_indices with all false
-  std::vector<bool> lib_indices(n, false);
-  std::vector<bool> pred_indices(n, false);
-
-  // Convert lib and pred (1-based in R) to 0-based indices and set corresponding positions to true
-  int libsize_int = lib.size();
-  for (int i = 0; i < libsize_int; ++i) {
-    lib_indices[lib[i] - 1] = true; // Convert to 0-based index
-  }
-  int predsize_int = pred.size();
-  for (int i = 0; i < predsize_int; ++i) {
-    pred_indices[pred[i] - 1] = true; // Convert to 0-based index
-  }
-
-  // /* Aligned with the previous implementation,
-  //  * now deprecated in the source C++ code.
-  //  *  ----- Wenbo Lv, written on 2025.02.10
-  //  */
-  // for (int i = 0, i < (Ex - 1) * tau, ++i){
-  //   lib_indices[lib[i] - 1] = false;
-  //   pred_indices[pred[i] - 1] = false;
-  // }
-
-  // /* Do not uncomment those codes;
-  //  * it's the previous implementation using `std::vector<std::pair<int, int>>`  input for lib and pred,
-  //  * kept for reference. ----- Wenbo Lv, written on 2025.02.09
-  //  */
-  // // Setup pred_indices
-  // std::vector<bool> pred_indices(n, false);
-  // for (const auto& p : pred) {
-  //   int row_start = p.first + (Ex - 1) * tau;
-  //   int row_end = p.second;
-  //   if (row_end > row_start && row_start >= 0 && row_end < n) {
-  //     std::fill(pred_indices.begin() + row_start, pred_indices.begin() + row_end + 1, true);
-  //   }
-  // }
-  //
-  // // Setup lib_indices
-  // std::vector<bool> lib_indices(n, false);
-  // for (const auto& l : lib) {
-  //   int row_start = l.first + (Ex - 1) * tau;
-  //   int row_end = l.second;
-  //   if (row_end > row_start && row_start >= 0 && row_end < n) {
-  //     std::fill(lib_indices.begin() + row_start, lib_indices.begin() + row_end + 1, true);
-  //   }
-  // }
-
-  int max_lib_size = std::accumulate(lib_indices.begin(), lib_indices.end(), 0); // Maximum lib size
   std::vector<int> possible_lib_indices;
-  for (int i = 0; i < n; ++i) {
-    if (lib_indices[i]) {
-      possible_lib_indices.push_back(i);
-    }
+  for (size_t i = 0; i < lib.size(); ++i) {
+    possible_lib_indices.push_back(lib[i]-1);
+  }
+  int max_lib_size = static_cast<int>(possible_lib_indices.size()); // Maximum lib size
+
+  std::vector<bool> pred_indices(n, false);
+  for (size_t i = 0; i < pred.size(); ++i) {
+    // // Do not strictly exclude spatial units with embedded state-space vectors containing NaN values from participating in cross mapping.
+    // if (!checkOneDimVectorHasNaN(x_vectors[pred[i] - 1])){
+    //   pred_indices[pred[i] - 1] = true;
+    // }
+    pred_indices[pred[i] - 1] = true; // Convert to 0-based index
   }
 
   std::vector<int> unique_lib_sizes(lib_sizes.begin(), lib_sizes.end());
@@ -396,81 +428,112 @@ std::vector<std::vector<double>> SCPCM4Lattice(
   //                [&](int size) { return std::max(size, Ex + 2); });
 
   // Remove duplicates
+  std::sort(unique_lib_sizes.begin(), unique_lib_sizes.end());
   unique_lib_sizes.erase(std::unique(unique_lib_sizes.begin(), unique_lib_sizes.end()), unique_lib_sizes.end());
 
   // Initialize the result container
   std::vector<PartialCorRes> x_xmap_y;
 
-  // Sequential version of the for loop
-  // for (int lib_size : unique_lib_sizes) {
-  //   RcppThread::Rcout << "lib_size: " << lib_size << "\n";
-  //   auto results = SCPCMSingle4Lattice(
-  //     x_vectors,
-  //     y,
-  //     controls,
-  //     nb_vec,
-  //     lib_indices,
-  //     lib_size,
-  //     max_lib_size,
-  //     possible_lib_indices,
-  //     pred_indices,
-  //     conEs,
-  //     contaus,
-  //     b,
-  //     simplex,
-  //     theta,
-  //     cumulate
-  //   );
-  //   x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
-  // }
-
-  // Perform the operations using RcppThread
-  if (progressbar) {
-    RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
-    RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
-      int lib_size = unique_lib_sizes[i];
-      auto results = SCPCMSingle4Lattice(
-        x_vectors,
-        y,
-        controls,
-        nb_vec,
-        lib_indices,
-        lib_size,
-        max_lib_size,
-        possible_lib_indices,
-        pred_indices,
-        conEs,
-        contaus,
-        b,
-        simplex,
-        theta,
-        cumulate
-      );
-      x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
-      bar++;
-    }, threads_sizet);
+  if (parallel_level == 0){
+    // Iterate over each library size
+    if (progressbar) {
+      RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
+      for (int lib_size : unique_lib_sizes) {
+        auto results = SCPCMSingle4Lattice(
+          x_vectors,
+          y,
+          controls,
+          nb_vec,
+          lib_size,
+          max_lib_size,
+          possible_lib_indices,
+          pred_indices,
+          conEs,
+          contaus,
+          bs,
+          simplex,
+          theta,
+          threads_sizet,
+          parallel_level,
+          cumulate
+        );
+        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
+        bar++;
+      }
+    } else {
+      for (int lib_size : unique_lib_sizes) {
+        auto results = SCPCMSingle4Lattice(
+          x_vectors,
+          y,
+          controls,
+          nb_vec,
+          lib_size,
+          max_lib_size,
+          possible_lib_indices,
+          pred_indices,
+          conEs,
+          contaus,
+          bs,
+          simplex,
+          theta,
+          threads_sizet,
+          parallel_level,
+          cumulate
+        );
+        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
+      }
+    }
   } else {
-    RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
-      int lib_size = unique_lib_sizes[i];
-      auto results = SCPCMSingle4Lattice(
-        x_vectors,
-        y,
-        controls,
-        nb_vec,
-        lib_indices,
-        lib_size,
-        max_lib_size,
-        possible_lib_indices,
-        pred_indices,
-        conEs,
-        contaus,
-        b,
-        simplex,
-        theta,
-        cumulate
-      );
-      x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
-    }, threads_sizet);
+    // Perform the operations using RcppThread
+    if (progressbar) {
+      RcppThread::ProgressBar bar(unique_lib_sizes.size(), 1);
+      RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
+        int lib_size = unique_lib_sizes[i];
+        auto results = SCPCMSingle4Lattice(
+          x_vectors,
+          y,
+          controls,
+          nb_vec,
+          lib_size,
+          max_lib_size,
+          possible_lib_indices,
+          pred_indices,
+          conEs,
+          contaus,
+          bs,
+          simplex,
+          theta,
+          threads_sizet,
+          parallel_level,
+          cumulate
+        );
+        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
+        bar++;
+      }, threads_sizet);
+    } else {
+      RcppThread::parallelFor(0, unique_lib_sizes.size(), [&](size_t i) {
+        int lib_size = unique_lib_sizes[i];
+        auto results = SCPCMSingle4Lattice(
+          x_vectors,
+          y,
+          controls,
+          nb_vec,
+          lib_size,
+          max_lib_size,
+          possible_lib_indices,
+          pred_indices,
+          conEs,
+          contaus,
+          bs,
+          simplex,
+          theta,
+          threads_sizet,
+          parallel_level,
+          cumulate
+        );
+        x_xmap_y.insert(x_xmap_y.end(), results.begin(), results.end());
+      }, threads_sizet);
+    }
   }
 
   // Group by the first int and store second and third values as pairs
